package amine.bou.DB

import amine.bou.models.HabitOccurrenceView
import amine.bou.models.HabitView
import amine.bou.models.toView
import io.ktor.server.plugins.*
import org.jetbrains.exposed.sql.and
import org.jetbrains.exposed.sql.transactions.transaction
import java.time.LocalDate

class DB {
    fun createHabit(request: HabitView): HabitView {
        return transaction {
            val h = Habit.new {
                name = request.name
                color = request.color
                recurrence = request.recurrence
            }
            h.toView()
        }
    }

    fun getAllHabits(): List<HabitView> {
        return transaction {
            return@transaction Habit.all().map {it.toView()}.toList()
        }
    }

    fun getOneHabit(id: Int): HabitView {
        return transaction {
            val habit = Habit.findById(id) ?: throw NotFoundException()
            return@transaction habit.toView()
        }
    }

    fun deleteHabit(id: Int) {
        return transaction {
            return@transaction Habit.findById(id)?.delete() ?: throw NotFoundException()
        }
    }

    fun updateHabit(id: Int, request: HabitView): HabitView {
        return transaction {
            val habit = Habit.findById(id) ?: throw NotFoundException()

            if (habit.color != request.color) {
                habit.color = request.color
            }

            if (habit.name != request.name) {
                habit.name = request.name
            }

            if (habit.recurrence != request.recurrence) {
                habit.recurrence = request.recurrence
            }

            // TODO: Handle occurrences on recurrence change ?

            return@transaction habit.toView()
        }
    }

    fun createOccurrence(habitId: Int, request: HabitOccurrenceView): HabitOccurrenceView {
        return transaction {
            val h = Habit.findById(habitId) ?: throw NotFoundException()
            
            val o = HabitOccurrence.new {
                occurence = LocalDate.parse(request.date)
                habit = h
            }

            // TODO: maybe handle streaks
            return@transaction o.toView()
        }
    }

    fun deleteOccurrence(habitId: Int, id: Long): Any {
        return transaction {
            val h = Habit.findById(habitId) ?: throw NotFoundException()

            return@transaction HabitOccurrence.find { HabitOccurrences.id eq id and (HabitOccurrences.habit eq habitId) }.firstOrNull()?.delete() ?: throw NotFoundException()
        }
    }
}